import numpy as np
import pandas as pd
import stan
import arviz as az
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb


def calcMAPEst(samples, a = 0.05, n_points = 1000):
    """　MAP推定値とMAP推定値における推定確率密度関数の値を求める
         samples:    MAP推定値を求めるサンプル
         a:　　　　　サンプルの左右a/2を除く。デフォルト値=0.05
         n_points:   KDEによる確率密度関数値を求める点の数。
         　　　　　　デフォルト値=1000
    """
    import numpy as np
    import scipy.stats as ss

    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord) 
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]
    #
    #   MAP推定値とMAP推定値における確率密度関数値
    #
    return MAP_Est, est_pdf[map_idx]    

finnm = input('Input Data File (*.xlsx) = ')

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

foutnm = input('Output Text File (*.txt) = ')
fout = open(foutnm, 'w')
fout.write('Input Data File = {}\n'.format(finnm))

print('data[0] =\n', data[0])
St = data[1:,1]
Res = data[1:,2]
for v in zip(St,Res):
    print(v)

N = len(St)

def f_init():
    return {'v_mu':np.median(St)/100.0,
            'v_sgm': np.std(St) / 100.0}  

Data = {'N':N, 'St':St, 'Res':Res}

with open('pm2cat.stan', 'r') as fstan:
    sm = stan.build(fstan.read(), data = Data)
fit = sm.sample(num_samples = 5000, num_chains = 4,
                init = [f_init()]*4)


i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data, var_names = ['mu', 'sgm'])
print('Summary =\n',smry)
fout.write(f'\nSummary:\n{smry}\n')

az.plot_trace(i_data, var_names = ['mu', 'sgm'])  
plt.show()

d_fm = fit.to_frame()

map_mu = calcMAPEst(d_fm['mu'])[0]
map_sgm = calcMAPEst(d_fm['sgm'])[0]

sb.kdeplot(d_fm['mu'])
plt.title(f'Posterior distribution of $\mu$,  MAP = {map_mu:.2f}')
plt.xlabel('$\mu$', fontsize = 14)
plt.show()

sb.kdeplot(d_fm['sgm'])
plt.title(f'Posterior distribution of $\sigma$,  MAP = {map_sgm:.2f}')
plt.xlabel('$\sigma$', fontsize = 14)
plt.show()

min_phi = np.min(St)
max_phi = np.max(St)
phis = np.linspace(min_phi, max_phi, 1000)

psis = ss.norm.cdf(phis, map_mu, map_sgm)
    
p_st = []
p_res = []
for i in range(0, N):
    p_st.append(St[i])
    if Res[i] == 1:
        p_res.append(0.1 * ss.uniform.rvs())
    else:
        p_res.append(1.0 - 0.1 * ss.uniform.rvs())
                            
plt.title("Psychometric function   PSE = {0:.5}   JND = {1:.5}".
          format(map_mu, map_sgm * 0.67449))
plt.plot(phis, psis, 'b', linewidth = 5)
plt.plot(p_st, p_res, 'go')
plt.xlabel('$S_{comp}$', fontsize = 14)
plt.legend(['PM', 'Data'], loc = 2)
plt.show()

fout.close()
print()
print(foutnm + ' was saved.')
print()
